@@ -1,15 +1,17 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
-from ._common import WarningTestMixin, NotAValue
+from ._common import NotAValue
 
 import calendar
 from datetime import datetime, date, timedelta
 import unittest
 
+import pytest
+
 from dateutil.relativedelta import relativedelta, MO, TU, WE, FR, SU
 
 
-class RelativeDeltaTest(WarningTestMixin, unittest.TestCase):
+class RelativeDeltaTest(unittest.TestCase):
     now = datetime(2003, 9, 17, 20, 54, 47, 282310)
     today = date(2003, 9, 17)
 
@@ -117,11 +119,19 @@ class RelativeDeltaTest(WarningTestMixin, unittest.TestCase):
         self.assertEqual(self.today+relativedelta(day=31, weekday=FR(-1)),
                          date(2003, 9, 26))
 
+    def testLastDayOfFebruary(self):
+        self.assertEqual(date(2021, 2, 1) + relativedelta(day=31),
+                         date(2021, 2, 28))
+
+    def testLastDayOfFebruaryLeapYear(self):
+        self.assertEqual(date(2020, 2, 1) + relativedelta(day=31),
+                         date(2020, 2, 29))
+
     def testNextWednesdayIsToday(self):
         self.assertEqual(self.today+relativedelta(weekday=WE),
                          date(2003, 9, 17))
 
-    def testNextWenesdayNotToday(self):
+    def testNextWednesdayNotToday(self):
         self.assertEqual(self.today+relativedelta(days=+1, weekday=WE),
                          date(2003, 9, 24))
 
@@ -371,25 +381,25 @@ class RelativeDeltaTest(WarningTestMixin, unittest.TestCase):
     def testRelativeDeltaFractionalAbsolutes(self):
         # Fractional absolute values will soon be unsupported,
         # check for the deprecation warning.
-        with self.assertWarns(DeprecationWarning):
+        with pytest.warns(DeprecationWarning):
             relativedelta(year=2.86)
 
-        with self.assertWarns(DeprecationWarning):
+        with pytest.warns(DeprecationWarning):
             relativedelta(month=1.29)
 
-        with self.assertWarns(DeprecationWarning):
+        with pytest.warns(DeprecationWarning):
             relativedelta(day=0.44)
 
-        with self.assertWarns(DeprecationWarning):
+        with pytest.warns(DeprecationWarning):
             relativedelta(hour=23.98)
 
-        with self.assertWarns(DeprecationWarning):
+        with pytest.warns(DeprecationWarning):
             relativedelta(minute=45.21)
 
-        with self.assertWarns(DeprecationWarning):
+        with pytest.warns(DeprecationWarning):
             relativedelta(second=13.2)
 
-        with self.assertWarns(DeprecationWarning):
+        with pytest.warns(DeprecationWarning):
             relativedelta(microsecond=157221.93)
 
     def testRelativeDeltaFractionalRepr(self):
@@ -474,7 +484,7 @@ class RelativeDeltaTest(WarningTestMixin, unittest.TestCase):
 
         self.assertEqual(rd1.normalized(), relativedelta(days=2, hours=18))
 
-        # Equvalent to (days=1, hours=11, minutes=31, seconds=12)
+        # Equivalent to (days=1, hours=11, minutes=31, seconds=12)
         rd2 = relativedelta(days=1.48)
 
         self.assertEqual(rd2.normalized(),
